unit ellipsoide;

interface

uses
  Classes;

type

  TEllipsoide = class
  private
    fa : real; // demi grand axe en mtre
    fe : real; // excentricit
    fNom : string; // nom (abrg) de l'ellipsode
    fDescription : string; // description d'ellipsode
    fValid : boolean; // indique si les paramtres de l'ellipsode sont bien dfinis
    function GetB : real;
    procedure SetB (Value : real);
    function GetInvF : real;
    procedure SetInvF (Value : real);
  public
    property a : real read fa write fa;
    property e : real read fe write fe;
    property b : real read GetB write SetB;
    property Invf : real read GetInvF write SetInvF;
    property Nom : string read fNom write fNom;
    property Description : string read fDescription write fDescription;
    property Valid : boolean read fValid;
    constructor Create;
    destructor Destroy; override;
    procedure LoadFromStream (Stream : TStream);
    procedure SaveToStream (Stream : TStream);
    procedure Assign(Autre : TEllipsoide);
  end;

implementation

uses
  SysUtils,
  outils;

constructor TEllipsoide.Create;
begin
  inherited Create;
  fValid:=false;
end;

destructor TEllipsoide.Destroy;
begin
  inherited Destroy;
end;

function TEllipsoide.GetB : real;
begin
  result:=a*sqrt(1-sqr(e));
end;

procedure TEllipsoide.SetB (Value : real);
begin
  e:=sqrt(1-sqr(Value/a));
end;

function TEllipsoide.GetInvF : real;
begin
  result:=a/(a-b);
end;

procedure TEllipsoide.SetInvF (Value : real);
begin
  b:=a*(1-1/Value);
end;

procedure TEllipsoide.LoadFromStream (Stream : TStream);
var
  Parametres : TstringList;
begin
  fValid:=false;
  Parametres:=TStringList.Create;
  try
    DetailParam(FinLigne(Stream),Parametres);
    if Parametres.Count<>4 then
      Raise ERangeError.Create('Nombre incorrecte de paramtre lors de la lecture d''un ellipsode');
    Nom:=Parametres[0];
    Description:=Parametres[1];
    a:=StrToFloat(Parametres[2]);
    e:=StrToFloat(Parametres[3]);
    fValid:=true;
  finally
    Parametres.Free;
  end;
end;

procedure TEllipsoide.SaveToStream (Stream : TStream);
var
  S : string;
begin
  S:=Format('Ellipsoide="%s" "%s" %.12g %.12g',[Nom, Description, a, e]);
  S:=S+#13#10;
  Stream.Write(S[1],length(S));
end;

procedure TEllipsoide.Assign(Autre : TEllipsoide);
begin
  Nom:=Autre.Nom;
  Description:=Autre.Description;
  a:=Autre.a;
  e:=Autre.e;
end;  


end.
